from typing import OrderedDict
import chromeRequests
import requests
import time

session = chromeRequests.session()

response = session.get("https://www.google.com/", headers=OrderedDict(
    {"User-Agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/80.0.3987.132 Safari/537.36"}))
print(response.status_code)
