from setuptools import setup, find_packages


setup(
    name='chromeRequests',
    version='0.2.3',
    author="Miro",
    license='MIT',
    packages=find_packages(),
    install_requires=[
        'scikit-learn',
    ],
    # include go folder
    package_data={
        'chromeRequests': ['go/compiled/library.so'],
    },
    include_package_data=True,
)
